@@ -3,13 +3,14 @@ package Crypt::KeyWrap;
 use strict;
 use warnings;
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 
 use Exporter 'import';
 our %EXPORT_TAGS = ( all => [qw(aes_key_wrap aes_key_unwrap gcm_key_wrap gcm_key_unwrap pbes2_key_wrap pbes2_key_unwrap ecdh_key_wrap ecdh_key_unwrap ecdhaes_key_wrap ecdhaes_key_unwrap rsa_key_wrap rsa_key_unwrap)] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
 
+use Carp;
 use Crypt::Mode::ECB;
 use Crypt::AuthEnc::GCM qw(gcm_encrypt_authenticate gcm_decrypt_verify);
 use Crypt::PRNG qw(random_bytes);
@@ -50,12 +51,12 @@ sub aes_key_wrap {
 
   my ($A, $B, $P, $R);
 
-  die "aes_key_wrap: no KEK"     unless defined $kek;
-  die "aes_key_wrap: no PT data" unless defined $pt_data;
+  croak "aes_key_wrap: no KEK"     unless defined $kek;
+  croak "aes_key_wrap: no PT data" unless defined $pt_data;
   my $klen = length $kek;
-  die "aes_key_wrap: invalid KEK length" unless $klen == 16 || $klen == 24 || $klen == 32;
-  die "aes_key_wrap: cipher must be AES or DES_EDE" unless $cipher eq 'AES' || $cipher eq 'DES_EDE';
-  die "aes_key_wrap: padding not allowed with DES_EDE" if $padding && $cipher eq 'DES_EDE';
+  croak "aes_key_wrap: invalid KEK length" unless $klen == 16 || $klen == 24 || $klen == 32;
+  croak "aes_key_wrap: cipher must be AES or DES_EDE" unless $cipher eq 'AES' || $cipher eq 'DES_EDE';
+  croak "aes_key_wrap: padding not allowed with DES_EDE" if $padding && $cipher eq 'DES_EDE';
 
   my $ECB = Crypt::Mode::ECB->new($cipher, 0);
   my $blck = $cipher eq 'DES_EDE' ? 4 : 8; # semiblock size in bytes, for AES 8, for 3DES 4
@@ -63,7 +64,7 @@ sub aes_key_wrap {
   my $IV = pack("H*", "A6" x $blck);
   my $len = length $pt_data;
   if ($len % $blck > 0) {
-    die "aes_key_wrap: pt_data length not multiply of $blck" if !$padding;
+    croak "aes_key_wrap: pt_data length not multiply of $blck" if !$padding;
     $pt_data .= chr(0) x ($blck - ($len % $blck));
     $IV = pack("H*", "A65959A6") . pack("N", $len);
   }
@@ -100,12 +101,12 @@ sub aes_key_unwrap {
 
   my ($A, $B, $C, $P, $R);
 
-  die "aes_key_unwrap: no KEK"     unless defined $kek;
-  die "aes_key_unwrap: no CT data" unless defined $ct_data;
+  croak "aes_key_unwrap: no KEK"     unless defined $kek;
+  croak "aes_key_unwrap: no CT data" unless defined $ct_data;
   my $klen = length $kek;
-  die "aes_key_unwrap: invalid KEK length" unless $klen == 16 || $klen == 24 || $klen == 32;
-  die "aes_key_unwrap: cipher must be AES or DES_EDE" unless $cipher eq 'AES' || $cipher eq 'DES_EDE';
-  die "aes_key_unwrap: padding not allowed with DES_EDE" if $padding && $cipher eq 'DES_EDE';
+  croak "aes_key_unwrap: invalid KEK length" unless $klen == 16 || $klen == 24 || $klen == 32;
+  croak "aes_key_unwrap: cipher must be AES or DES_EDE" unless $cipher eq 'AES' || $cipher eq 'DES_EDE';
+  croak "aes_key_unwrap: padding not allowed with DES_EDE" if $padding && $cipher eq 'DES_EDE';
 
   my $ECB = Crypt::Mode::ECB->new($cipher, 0);
   my $blck = $cipher eq 'DES_EDE' ? 4 : 8; # semiblock size in bytes, for AES 8, for 3DES 4
@@ -144,10 +145,10 @@ sub aes_key_unwrap {
     my $n = unpack("N", substr($A, 4, 4));
     my $z = length($rv) - $n;
     my $tail = unpack("H*", substr($rv, -$z));
-    die "aes_key_unwrap: invalid data" unless $tail eq "00"x$z;
+    croak "aes_key_unwrap: invalid data" unless $tail eq "00"x$z;
     return substr($rv, 0, $n);
   }
-  die "aes_key_unwrap: unexpected data [$cipher/$A_hex]";
+  croak "aes_key_unwrap: unexpected data [$cipher/$A_hex]";
 }
 
 # AES GCM KW - https://tools.ietf.org/html/rfc7518#section-4.7
@@ -179,7 +180,7 @@ sub pbes2_key_wrap {
     my $ct_data = aes_key_wrap($aes_key, $pt_data);
     return $ct_data;
   }
-  die "pbes2_key_wrap: invalid alg '$alg'";
+  croak "pbes2_key_wrap: invalid alg '$alg'";
   return undef;
 }
 
@@ -193,7 +194,7 @@ sub pbes2_key_unwrap {
     my $pt_data = aes_key_unwrap($aes_key, $ct_data);
     return $pt_data;
   }
-  die "pbes2_key_unwrap: invalid alg '$alg'";
+  croak "pbes2_key_unwrap: invalid alg '$alg'";
   return undef;
 }
 
@@ -203,25 +204,25 @@ sub pbes2_key_unwrap {
 
 sub rsa_key_wrap {
   my ($kek_public, $pt_data, $alg) = @_;
-  die "rsa_key_wrap: no Crypt::PK::RSA" unless ref $kek_public eq 'Crypt::PK::RSA';
+  croak "rsa_key_wrap: no Crypt::PK::RSA" unless ref $kek_public eq 'Crypt::PK::RSA';
   my ($padding, $hash_name);
   if    ($alg eq 'RSA-OAEP')     { ($padding, $hash_name) = ('oaep', 'SHA1') }
   elsif ($alg eq 'RSA-OAEP-256') { ($padding, $hash_name) = ('oaep', 'SHA256') }
   elsif ($alg eq 'RSA1_5')       { $padding = 'v1.5' }
-  die "rsa_key_wrap: invalid algorithm '$alg'" unless $padding;
+  croak "rsa_key_wrap: invalid algorithm '$alg'" unless $padding;
   my $ct_data = $kek_public->encrypt($pt_data, $padding, $hash_name);
   return $ct_data;
 }
 
 sub rsa_key_unwrap {
   my ($kek_private, $ct_data, $alg) = @_;
-  die "rsa_key_unwrap: no Crypt::PK::RSA" unless ref $kek_private eq 'Crypt::PK::RSA';
-  die "rsa_key_unwrap: no private key" unless $kek_private->is_private;
+  croak "rsa_key_unwrap: no Crypt::PK::RSA" unless ref $kek_private eq 'Crypt::PK::RSA';
+  croak "rsa_key_unwrap: no private key" unless $kek_private->is_private;
   my ($padding, $hash_name);
   if    ($alg eq 'RSA-OAEP')     { ($padding, $hash_name) = ('oaep', 'SHA1') }
   elsif ($alg eq 'RSA-OAEP-256') { ($padding, $hash_name) = ('oaep', 'SHA256') }
   elsif ($alg eq 'RSA1_5')       { $padding = 'v1.5' }
-  die "rsa_key_unwrap: invalid algorithm '$alg'" unless $padding;
+  croak "rsa_key_unwrap: invalid algorithm '$alg'" unless $padding;
   my $pt_data = $kek_private->decrypt($ct_data, $padding, $hash_name);
   return $pt_data;
 }
@@ -250,7 +251,7 @@ sub _concat_kdf {
 
 sub ecdh_key_wrap {
   my ($kek_public, $enc, $apu, $apv) = @_;
-  die "ecdh_key_wrap: no Crypt::PK::ECC" unless ref $kek_public eq 'Crypt::PK::ECC';
+  croak "ecdh_key_wrap: no Crypt::PK::ECC" unless ref $kek_public eq 'Crypt::PK::ECC';
   my $encryption_key_size = 256;
   if ($enc =~ /^A(128|192|256)CBC-HS/) {
     $encryption_key_size = $1*2;
@@ -266,8 +267,8 @@ sub ecdh_key_wrap {
 
 sub ecdh_key_unwrap {
   my ($kek_private, $enc, $epk, $apu, $apv) = @_;
-  die "ecdh_key_unwrap: no Crypt::PK::ECC" unless ref $kek_private eq 'Crypt::PK::ECC';
-  die "ecdh_key_unwrap: no private key" unless $kek_private->is_private;
+  croak "ecdh_key_unwrap: no Crypt::PK::ECC" unless ref $kek_private eq 'Crypt::PK::ECC';
+  croak "ecdh_key_unwrap: no private key" unless $kek_private->is_private;
   my $encryption_key_size = 256;
   if ($enc =~ /^A(128|192|256)CBC-HS/) {
     $encryption_key_size = $1*2;
@@ -283,7 +284,7 @@ sub ecdh_key_unwrap {
 
 sub ecdhaes_key_wrap {
   my ($kek_public, $pt_data, $alg, $apu, $apv) = @_;
-  die "ecdhaes_key_wrap: no Crypt::PK::ECC" unless ref $kek_public eq 'Crypt::PK::ECC';
+  croak "ecdhaes_key_wrap: no Crypt::PK::ECC" unless ref $kek_public eq 'Crypt::PK::ECC';
   my $encryption_key_size = 256;
   if ($alg =~ /^ECDH-ES\+A(128|192|256)KW$/) {
     $encryption_key_size = $1;
@@ -296,8 +297,8 @@ sub ecdhaes_key_wrap {
 
 sub ecdhaes_key_unwrap {
   my ($kek_private, $ct_data, $alg, $epk, $apu, $apv) = @_;
-  die "ecdhaes_key_unwrap: no Crypt::PK::ECC" unless ref $kek_private eq 'Crypt::PK::ECC';
-  die "ecdhaes_key_unwrap: no private key" unless $kek_private->is_private;
+  croak "ecdhaes_key_unwrap: no Crypt::PK::ECC" unless ref $kek_private eq 'Crypt::PK::ECC';
+  croak "ecdhaes_key_unwrap: no private key" unless $kek_private->is_private;
   my $encryption_key_size = 256;
   if ($alg =~ /^ECDH-ES\+A(128|192|256)KW$/) {
     $encryption_key_size = $1;
